"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ruleName = exports.messages = void 0;
const caniuse_lite_1 = require("caniuse-lite");
const postcss_selector_parser_1 = __importDefault(require("postcss-selector-parser"));
const stylelint_1 = __importDefault(require("stylelint"));
const browserslist_1 = __importDefault(require("browserslist"));
const ruleName = 'plugin/no-unsupported-css-pseudo';
exports.ruleName = ruleName;
const messages = stylelint_1.default.utils.ruleMessages(ruleName, {
    accepted: () => '',
    rejected: (details, browser) => {
        return `css pseudo "${details}" is not supported your browserslist (${browser})`;
    },
});
exports.messages = messages;
const CSS_PESUDO_MAP = new Map();
const pseudoReg = /(?<=(:|::))(.+?)(?=( |\())/;
for (const key in caniuse_lite_1.features) {
    if (key.includes('css')) {
        const cssFeature = (0, caniuse_lite_1.feature)(caniuse_lite_1.features[key]);
        if (cssFeature.title.includes(':')) {
            const matchRes = cssFeature.title.match(pseudoReg);
            if (matchRes?.length) {
                const cssPseudoSelectorkey = `${matchRes[1].trim()}${(matchRes[0]).trim()}`;
                CSS_PESUDO_MAP.set(cssPseudoSelectorkey, (0, caniuse_lite_1.feature)(caniuse_lite_1.features[key]));
            }
        }
    }
}
const ruleFunction = (_primaryOption, { browsers }) => {
    return (postCssRoot, postCssResult) => {
        const validOptions = stylelint_1.default.utils.validateOptions(postCssResult, ruleName);
        if (!validOptions)
            return;
        postCssRoot.walkRules((node) => {
            (0, postcss_selector_parser_1.default)().astSync(node.selector).walkPseudos((pseudoNode) => {
                const pseudoBrowserSupportInfo = CSS_PESUDO_MAP.get(pseudoNode.value);
                if (!pseudoBrowserSupportInfo)
                    return;
                const currentBrowserSupportInfo = (0, browserslist_1.default)(browsers);
                const formatCurrentBrowserSupportInfo = currentBrowserSupportInfo.reduceRight((prev, cur) => {
                    const [browser, version] = cur.split(' ');
                    if (browser in prev) {
                        prev[browser].versions.push(version);
                    }
                    else {
                        prev[browser] = {
                            versions: [version],
                            minVersion: version,
                        };
                    }
                    return prev;
                }, {});
                Object.keys(formatCurrentBrowserSupportInfo).find((browser) => {
                    const { minVersion } = formatCurrentBrowserSupportInfo[browser];
                    if (pseudoBrowserSupportInfo.stats[browser][minVersion] !== 'y') {
                        stylelint_1.default.utils.report({
                            ruleName,
                            result: postCssResult,
                            message: messages.rejected(pseudoNode.value, browsers.join(', ')),
                            node,
                        });
                    }
                    return true;
                });
                // for (const browser in formatCurrentBrowserSupportInfo) {
                //   if (Object.prototype.hasOwnProperty.call(formatCurrentBrowserSupportInfo, browser)) {
                //     const { minVersion } = formatCurrentBrowserSupportInfo[browser]
                //     if (pseudoBrowserSupportInfo.stats[browser][minVersion] !== 'y') {
                //       stylelint.utils.report({
                //         ruleName,
                //         result: postCssResult,
                //         message: messages.rejected(pseudoNode.value, browsers.join(',')),
                //         node,
                //       })
                //     }
                //   }
                // }
            });
        });
    };
};
ruleFunction.ruleName = ruleName;
ruleFunction.messages = messages;
exports.default = stylelint_1.default.createPlugin(ruleName, ruleFunction);
